from detectron2 import model_zoo
from detectron2.config import get_cfg
from detectron2.utils.logger import setup_logger
from detectron2.evaluation import COCOEvaluator, inference_on_dataset
from detectron2.data.datasets import register_coco_instances
from detectron2.data import build_detection_test_loader, MetadataCatalog, DatasetCatalog
from detectron2.engine import DefaultTrainer
setup_logger()

from dicts import seq_dict, qp_dict

import sys, os, subprocess
import torch

if len(sys.argv) != 5:
    print("Usage detection_work.py <test_name> <base_dir> <gt_file> <out_dir>")
    sys.exit(-1)
test_name = sys.argv[1]
base_dir  = sys.argv[2]
gt_dir    = sys.argv[3]
out_dir   = sys.argv[4]

if not os.path.isdir(out_dir):
    os.mkdir(out_dir)

class_list = [
    "Person", "Bicycle", "Car", "Motorcycle", "Airplane", "Bus", "Train", "Truck",
    "Boat", "Traffic light", "Fire hydrant", "Wheel", "Suit", "Man", "Bird", "Cat",
    "Dog", "Horse", "Sheep", "Cattle", "Elephant", "Bear", "Zebra", "Giraffe",
    "Backpack", "Umbrella", "Handbag", "Tie", "Suitcase", "Woman", "Ski",
    "Snowboard", "Football", "Kite", "Baseball bat", "Baseball glove", "Skateboard",
    "Surfboard", "Tennis racket", "Bottle", "Wine glass", "Coffee cup", "Fork",
    "Knife", "Spoon", "Bowl", "Banana", "Apple", "Sandwich", "Orange", "Broccoli",
    "Carrot", "Hot dog", "Pizza", "Bagel", "Cake", "Chair", "Couch",
    "Common sunflower", "Bed", "Houseplant", "Toilet", "Television", "Laptop",
    "Computer mouse", "Bicycle wheel", "Computer keyboard", "Mobile phone",
    "Microwave oven", "Oven", "Boy", "Sink", "Refrigerator", "Book", "Clock",
    "Vase", "Scissors", "Teddy bear", "Girl", "Human face",
]


print("Working on %s...\n" % test_name)

#cfg set method
def model_cfg_set(model_pth):
    cfg = get_cfg()
    cfg.merge_from_file(model_zoo.get_config_file("COCO-Detection/faster_rcnn_X_101_32x8d_FPN_3x.yaml"))
    cfg.MODEL.WEIGHTS = model_pth
    cfg.MODEL.ROI_HEADS.NUM_CLASSES = len(class_list)
#    cfg.MODEL.ROI_HEADS.SCORE_THRESH_TEST = 0.5
    cfg.TEST.DETECTIONS_PER_IMAGE = 200
    cfg.DATASETS.TRAIN = (dataset_name,)  # This is required to set the metadata format from the Trainer.
    cfg.OUTPUT_DIR = out_dir
#    cfg.MODEL.DEVICE = "cpu"
    return cfg

#get the model from cfg and set the trainer
#we calculate the AP using Trainer because it is fast and easy
model_path = "../models/detection.pkl"

torch.cuda.set_device(0)

for seq in seq_dict:
    cl, seq_name = seq_dict[seq]
    if test_name == "unc":
        QPs = [0]
    else:
        QPs = qp_dict[cl]
    gt_file = os.path.join(gt_dir, "%s.json" % seq_name)
    for qp in QPs:
        dataset_name = seq_name
        if qp > 0:
            dataset_name += "_qp%d" % qp
        img_dir = os.path.join(base_dir, dataset_name)

        cfg = model_cfg_set(model_path)
        register_coco_instances(dataset_name, {}, gt_file, img_dir)
        trainer = DefaultTrainer(cfg)
        trainer.resume_or_load(resume = False)

        # calculate the AP using COCOEvaluator
        evaluator = COCOEvaluator(dataset_name, cfg, False, output_dir = out_dir)
        val_loader = build_detection_test_loader(cfg, dataset_name)
        results = inference_on_dataset(trainer.model, val_loader, evaluator)

        print("==============================")
        print(f'AP={results["bbox"]["AP"]}')

        res_file = os.path.join(out_dir, "results_%s_%s.txt" % (test_name, dataset_name))
        with open(res_file, "w") as res:
            for metric in results["bbox"]:
                res.write("%10s = %8.6f\n" % (metric, results["bbox"][metric]))

    print("Done with %s!\n" % seq_name)

classes = ["AB", "C", "D"]
if "unc" in test_name:
    create_classwise = "python classwise_unc.py %s %s" % (base_dir, base_dir)
else:
    create_classwise = "python classwise.py %s %s" % (base_dir, base_dir)
subprocess.run(create_classwise, shell = True)
for c in classes:
     gt_file = os.path.join(gt_dir, "class%s.json" % c)
     num_qps = 1 if "unc" in test_name else 6
     for rp in range(num_qps):
         dataset_name = "class%s" % c
         if "unc" not in test_name:
             dataset_name += "_%d" % rp
        img_dir = os.path.join(base_dir, dataset_name)

        cfg = model_cfg_set(model_path)
        register_coco_instances(dataset_name, {}, gt_file, img_dir)
        trainer = DefaultTrainer(cfg)
        trainer.resume_or_load(resume = False)

        # calculate the AP using COCOEvaluator
        evaluator = COCOEvaluator(dataset_name, cfg, False, output_dir = out_dir)
        val_loader = build_detection_test_loader(cfg, dataset_name)
        results = inference_on_dataset(trainer.model, val_loader, evaluator)

        print("==============================")
        print(f'AP={results["bbox"]["AP"]}')

        res_file = os.path.join(out_dir, "results_%s_%s.txt" % (test_name, dataset_name))
        with open(res_file, "w") as res:
            for metric in results["bbox"]:
                res.write("%10s = %8.6f\n" % (metric, results["bbox"][metric]))

    print("Done with class %s!\n" % c)

print("Done with %s!\n" % test_name)

