from detectron2 import model_zoo
from detectron2.config import get_cfg
from detectron2.evaluation import COCOEvaluator, inference_on_dataset
from detectron2.data.datasets import register_coco_instances
from detectron2.data import build_detection_test_loader
from detectron2.engine import DefaultTrainer
import argparse, os
import csv

#cfg set method
def model_cfg_set(model_pth):
    cfg = get_cfg()
    cfg.merge_from_file(model_zoo.get_config_file("COCO-Detection/faster_rcnn_X_101_32x8d_FPN_3x.yaml"))
    cfg.MODEL.WEIGHTS = model_pth
    cfg.MODEL.ROI_HEADS.NUM_CLASSES = 3
    cfg.MODEL.ROI_HEADS.SCORE_THRESH_TEST = 0.5
    cfg.DATASETS.TRAIN = ("FLIR_val",)  # This is required to set the metadata format from the Trainer.
    return cfg


if __name__=="__main__":
    # define the model, datas
    parser = argparse.ArgumentParser()
    parser.add_argument('--image_path', type=str, default=None)
    parser.add_argument('--model_path', type=str, default=None)
    parser.add_argument('--coco_gt_path', type=str, default=None)
    parser.add_argument('--result_path', type=str, default=None)
    parser.add_argument('--result_file_name', type=str, default=None)
    parser.add_argument('--qp', type=int, default=None)

    args = parser.parse_args()
    model_path=args.model_path
    coco_gt_path=args.coco_gt_path
    image_path=args.image_path
    result_path=args.result_path
    result_file_name=args.result_file_name
    qp=args.qp

    # get the model from cfg and set the trainer
    # we calculate the AP using Trainer because it is fast and easy
    register_coco_instances("FLIR_val", {}, coco_gt_path, image_path)

    cfg = model_cfg_set(model_path)
    trainer = DefaultTrainer(cfg)
    trainer.resume_or_load(resume=False)

    # calculate the AP using COCOEvaluator
    evaluator = COCOEvaluator("FLIR_val", cfg, False, output_dir="output/")
    val_loader = build_detection_test_loader(cfg, "FLIR_val")
    results=inference_on_dataset(trainer.model, val_loader, evaluator)


    csv_fname=result_path+"/"+result_file_name+".csv"

    if os.path.isfile(os.path.join(result_path, csv_fname)) == False:
        f = open(csv_fname, 'w', encoding='utf-8', newline='')
        wr = csv.writer(f)
        wr.writerow(['fname', 'AP', 'AP50', 'AP75', 'APs', 'APm', 'APl', 'AP-person', 'AP-bicycle', 'AP-car'])
    else:
        f = open(csv_fname, 'a', encoding='utf-8', newline='')
        wr = csv.writer(f)
    wr.writerow([result_file_name + "_" + str(qp), results['bbox']['AP'],
                 results['bbox']['AP50'], results['bbox']['AP75'],
                 results['bbox']['APs'], results['bbox']['APm'],
                 results['bbox']['APl'], results['bbox']['AP-person'],
                 results['bbox']['AP-bicycle'], results['bbox']['AP-car']])
    f.close()