# SW requirement
# 1. cuda 11.1
# 2. pytorch 1.9.0
# 3. torchvision  0.10.0
# 3. detectorn2(<https://github.com/facebookresearch/detectron2/blob/master/INSTALL.md>)
# 4. opencv-python 4.5.4

import cv2
import os, argparse
from detectron2 import model_zoo
from detectron2.engine import DefaultPredictor
from detectron2.config import get_cfg
from detectron2.utils.visualizer import Visualizer
from detectron2.data import MetadataCatalog

## getting/setting the cfg and setting the model
def model_set(weight_path):
    cfg = get_cfg()

    model="COCO-Detection/faster_rcnn_X_101_32x8d_FPN_3x.yaml"

    cfg.merge_from_file(model_zoo.get_config_file(model))
    cfg.MODEL.ROI_HEADS.SCORE_THRESH_TEST = 0.5
    cfg.MODEL.ROI_HEADS.NUM_CLASSES = 3
    cfg.MODEL.WEIGHTS = weight_path

    return cfg

## predict a image from the model
def inference(cfg, img, metadata):
    predictor = DefaultPredictor(cfg)
    outputs = predictor(img)

    p_classes = outputs["instances"].pred_classes.tolist()

    if p_classes.__len__() != 0:
        p_draw = Visualizer(img[:, :, ::-1], metadata, scale=1)
        p_draw = p_draw.draw_instance_predictions(outputs["instances"].to("cpu"))
        p_draw = cv2.cvtColor(p_draw.get_image()[:, :, ::-1], cv2.COLOR_BGR2RGB)

        return p_draw


if __name__=="__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('--image_path', type=str, default=None)
    parser.add_argument('--weight_path', type=str, default=None)
    parser.add_argument('--result_path', type=str, default=None)

    args = parser.parse_args()
    image_path = args.image_path
    weight_path = args.weight_path
    result_path = args.result_path

    #extract the image name from the path
    image_name = os.path.basename(image_path)
    image_name = image_name[:-4]
    print(f'image name = {image_name}')

    #load a image
    img = cv2.imread(image_path)

    #set the model and metadata
    cfg = model_set(weight_path)
    metadata = MetadataCatalog.get(cfg.DATASETS.TRAIN[0])

    #predict a image from the model
    p_draw = inference(cfg, img, metadata)

    #save a inference image
    cv2.imwrite(os.path.join(result_path, image_name + "_inference.png"), p_draw)
