# Copyright (c) 2020-2021 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
# This file is covered by the license agreement found in the file “license.txt” in the root of this project.

#
# This script generates all results for object detection and segmentation on the 
# OpenImages dataset

set -e

#####################################################
# handle command line arguments
if [ "$#" -ne 1 ]; then
  echo "
This script generates object detection and segmentation tasks for anchors or proposed technologies. To reproduce anchors, run
  $0 anchor.cfg

To generate machine task results for technologies for VCM, run 
  $0 proposal.cfg
" 
  exit 0
else
  cfg_fname=$1  
fi

# read config file
source $cfg_fname
mkdir -p $output_dir

#####################################################

# calcuate bpp
echo "Calculating bpp for each compressed data"
python calculate_bpp.py $cfg_fname


# rescle the images to original resolution
echo "Rescaling the images to original resolution"
python rescale.py $cfg_fname


echo
echo "#################################################"
echo

# detection task
if echo $task | grep -w -q 'detection'; then
  echo "processing detection ..."
  bash gen_detection_output.sh $cfg_fname
  bash eval_detection.sh $cfg_fname
fi



echo
echo "#################################################"
echo

# segmentation task
if echo $task | grep -w -q 'segmentation'; then
  echo "processing segmentation ..."
  bash gen_segmentation_output.sh $cfg_fname
  bash eval_segmentation.sh $cfg_fname
fi

echo
echo "#################################################"
echo


# generate report csv and plots
echo "generating summary ..."
python gen_results.py $cfg_fname

echo 
echo 'Done!'

