# Copyright (c) 2020-2021 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
# This file is covered by the license agreement found in the file “license.txt” in the root of this project.

#
# this script calcuates bpp for object detection and segmentation tasks

import pandas as pd
import glob
import os
import cv2

import utils 

# read config file
cfg = utils.read_config()

QP_list = cfg.qps.split()
bitstream_dir = cfg.bitstream_dir
output_dir = cfg.output_dir

# image directory
img_dir = '../dataset/validation/validation'

# annotations
dataset_dir='../dataset'
annot_dir = os.path.join(dataset_dir, "annotations_5k")

input_img_list = []
if 'detection' in cfg.task.split(): 
  input_img_list.append(os.path.join(annot_dir, 'detection_validation_input_5k.lst'))
if 'segmentation' in cfg.task.split():
  input_img_list.append(os.path.join(annot_dir, 'segmentation_validation_input_5k.lst'))


output_data_file = os.path.join(output_dir, 'bpp_data.csv')
output_bpp_report_file  = os.path.join(output_dir, 'bpp_report.csv')


###############################################
bpp_data = []

# get validation image ids
val_img_ids = set()
for img_list_fname in input_img_list:
  with open(img_list_fname) as f:
    imgs = f.read().splitlines()
    ids = [os.path.splitext(x)[0] for x in imgs]
    val_img_ids = val_img_ids.union(ids)

for img_id in val_img_ids:
  img_fname = os.path.join(img_dir, img_id+'.jpg')
  img = cv2.imread(img_fname)
  H,W,_ = img.shape
  for QP in QP_list:
    bs_fname = glob.glob(os.path.join(bitstream_dir, str(QP), f'{img_id}.*'))[0]
    bs_size = os.path.getsize(bs_fname)
    bpp = bs_size * 8 / (W*H)
    bpp_data.append({ \
      'ImageID': img_id,
      'ImageWidth': W,
      'ImageHeight': H,
      'QP': QP,
      'BitstreamSize': bs_size,
      'Bpp': bpp })

bpp_data = pd.DataFrame(bpp_data)
bpp_data.to_csv(output_data_file, index=False)
   
# generate bpp for each task and for each QP

bpp_data = pd.read_csv(output_data_file)
bpp_report = []
#detection
for img_list in input_img_list:
  print('processing', img_list)
  with open(img_list, 'r') as f:
    img_id_list=[x[:-4] for x in f.read().splitlines()] #remove .png
  
  for QP in QP_list:
    sub_data = bpp_data.loc[bpp_data['ImageID'].isin(img_id_list) & (bpp_data['QP']==QP)]

    # this line calculate the average per image
    #avg_bpp = sub_data['Bpp'].mean()

    # calculate the average in total bits and total pixels
    total_bits = sub_data['BitstreamSize'].sum() * 8
    total_pixels = (sub_data['ImageHeight'] * sub_data['ImageWidth']).sum()
    avg_bpp = total_bits / total_pixels
   
    bpp_report.append({ \
      'ImageList': os.path.basename(img_list),
      'QP': QP,
      'bpp': avg_bpp})
    print(QP, avg_bpp)

bpp_report = pd.DataFrame(bpp_report)
bpp_report.to_csv(output_bpp_report_file, index=False)


