# Copyright (c) 2020-2021 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
# This file is covered by the license agreement found in the file “license.txt” in the root of this project.

# evaluation according to instruction from 
#https://github.com/tensorflow/models/blob/master/research/object_detection/g3doc/challenge_evaluation.md#object-detection-track

set -e

# load configuration
cfg_fname=$1  
source $cfg_fname


dataset_dir=$(realpath ../dataset)

anno_dir=$dataset_dir/annotations_5k
segmentation_mask_dir=$anno_dir/challenge_2019_validation_masks
BOUNDING_BOXES=$anno_dir/segmentation_validation_bbox_5k.csv
IMAGE_LABELS=$anno_dir/segmentation_validation_labels_5k.csv
INSTANCE_SEGMENTATIONS=$anno_dir/segmentation_validation_masks_5k.csv
LABEL_MAP=$anno_dir/coco_label_map.pbtxt

# get TF object_detection path
TF_od_dir=$(python -c "import object_detection,os;print(os.path.dirname(object_detection.__file__))")

if [ $eval_uncompressed_data == 1 ]; then
  qps="uncmprd $qps"
fi

# evaluation
#for QP in uncmprd QP_22 QP_27 QP_32 QP_37 QP_42 QP_47; do
for QP in $qps; do
  echo processing ... $QP 

  INPUT_PREDICTIONS=${output_dir}/output_segmentation_${QP}.txt.oid.txt
  OUTPUT_METRICS=${output_dir}/segmentation_${QP}_metric.txt

  sem -j+0 python -u $TF_od_dir/metrics/oid_challenge_evaluation.py \
      --input_annotations_boxes=${BOUNDING_BOXES} \
      --input_annotations_labels=${IMAGE_LABELS} \
      --input_class_labelmap=${LABEL_MAP} \
      --input_annotations_segm=${INSTANCE_SEGMENTATIONS} \
      --input_predictions=${INPUT_PREDICTIONS} \
      --output_metrics=${OUTPUT_METRICS} 

done

sem --wait

echo 
echo 'Evaluation results: '
echo 

#for QP in uncmprd QP_22 QP_27 QP_32 QP_37 QP_42 QP_47; do
for QP in $qps; do
  OUTPUT_METRICS=${output_dir}/segmentation_${QP}_metric.txt
  rslt=$(head $OUTPUT_METRICS -n 1 | cut -d , -f 2)
  echo $rslt
done

echo Done!
