# Copyright (c) 2020-2021 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
# This file is covered by the license agreement found in the file “license.txt” in the root of this project.

# rescale the input image to the original size
# for 100% resolution, crop is used
# for other resolutions, rescale is used

import pandas as pd
import subprocess
import os
import glob
import multiprocessing
import utils


# read config file
cfg = utils.read_config()

output_dir = cfg.output_dir
qps = cfg.qps.split()

dataset_dir='../dataset'
annot_dir = os.path.join(dataset_dir, "annotations_5k")

bpp_data_fname = os.path.join(output_dir, 'bpp_data.csv')
input_img_list = []
if 'detection' in cfg.task.split(): 
  input_img_list.append(os.path.join(annot_dir, 'detection_validation_input_5k.lst'))
if 'segmentation' in cfg.task.split():
  input_img_list.append(os.path.join(annot_dir, 'segmentation_validation_input_5k.lst'))

input_data_dir = cfg.recon_data_dir
output_data_dir = os.path.join(output_dir, 'rescaled')

data = pd.read_csv(bpp_data_fname)

# collect images to be process
img_fnames = []
for input_fname in input_img_list:
  with open(input_fname, 'r') as f:
    imgs = f.read().splitlines()

  img_fnames += imgs

img_fnames = list(set(img_fnames))
img_ids = sorted([os.path.splitext(x)[0] for x in img_fnames])


# get width and height
sub_data = data[(data['ImageID'].isin(img_ids)) & (data['QP']==qps[0])]

def process_item(item):
  input_fname, output_fname, width, height = item

  cmd = f'ffmpeg -y -hide_banner -loglevel error -i {input_fname} -vf "crop={width}:{height}" {output_fname}'

  p = subprocess.Popen(cmd, shell=True, stderr=subprocess.STDOUT, stdout=subprocess.PIPE)
  for line in p.stdout.readlines():
    print(line)



# get subdirectories
item_list = []
for qp_name in qps:
  print('rescalinng files in ', qp_name)

  output_dir = os.path.join(output_data_dir, qp_name)
  os.makedirs(output_dir, exist_ok=True)
  
  for index, row in sub_data.iterrows():
    img_id = row['ImageID']
    width = row['ImageWidth']
    height = row['ImageHeight']
    
    input_fname = os.path.join(input_data_dir, qp_name, img_id+'.png')
    output_fname =  os.path.join(output_dir, img_id+'.png')
    item_list.append((input_fname, output_fname, width, height))


#for item in item_list:
#  process_item(item)

# parallel processing
pool = multiprocessing.Pool(32)
result = pool.map(process_item, item_list)
  
# validate all images have been generated
print('validating rescale results')
for qp_name in qps:
  output_dir = os.path.join(output_data_dir, qp_name)
  output_imgs = glob.glob(os.path.join(output_dir, '*.png'))
  output_imgs = [os.path.basename(x) for x in output_imgs]
  # images that have not been processed
  print('The following images are not processed for ', qp_name)
  print(set(img_fnames) - set(output_imgs))


print('All Done')


