#
# this script calcuates bpp for object detection and segmentation tasks

import pandas as pd
import glob
import os
import cv2

import utils 

# read config file
cfg = utils.read_config()

QP_list = cfg.qps.split()
bitstream_dir = cfg.bitstream_dir
output_dir = cfg.output_dir

# image directory
img_dir = '../dataset/validation/object_det_seg'

# annotations
dataset_dir='../dataset'
annot_dir = os.path.join(dataset_dir, "annotations_tvd")
img_list = os.path.join(annot_dir, 'tvd_detection_input_new.lst')


output_data_file = os.path.join(output_dir, 'bpp_data.csv')
output_bpp_report_file  = os.path.join(output_dir, 'bpp_report.csv')


###############################################
bpp_data = []

with open(img_list, 'r') as f:
    img_files = [x.strip() for x in f.read().splitlines()]

for idx, img_fname in enumerate(img_files):
  img_id = img_fname[:-4]
  img_fname = os.path.join(img_dir, img_fname)

  img = cv2.imread(img_fname)
  H,W,_ = img.shape

  for QP in QP_list:
    bs_fname = glob.glob(os.path.join(bitstream_dir, QP, f'{img_id}.*'))[0]
    bs_size = os.path.getsize(bs_fname)
    bpp = bs_size * 8 / (W*H)
    bpp_data.append({ \
      'ImageID': img_id,
      'ImageWidth': W,
      'ImageHeight': H,
      'QP': QP,
      'BitstreamSize': bs_size,
      'Bpp': bpp })

bpp_data = pd.DataFrame(bpp_data)
bpp_data.to_csv(output_data_file, index=False)

# generate bpp for each task and for each QP
bpp_data = pd.read_csv(output_data_file)
bpp_report=[]
print('processing', img_list)

for QP in QP_list:
  sub_data = bpp_data.loc[(bpp_data['QP']==QP)]

  # calculate the average in total bits and total pixels
  total_bits = sub_data['BitstreamSize'].sum() * 8
  total_pixels = (sub_data['ImageHeight'] * sub_data['ImageWidth']).sum()
  avg_bpp = total_bits / total_pixels
  bpp_report.append({ \
      'ImageList': os.path.basename(img_list),
      'QP': QP,
      'bpp': avg_bpp})
  print(QP, avg_bpp)

bpp_report=pd.DataFrame(bpp_report)
bpp_report.to_csv(output_bpp_report_file, index=False)


