#
# this script calcuates average psnr and msssim scores for input images and reconstructed images for object tracking


import pandas as pd
import glob
import os
import cv2
import numpy as np
import torch
import pytorch_msssim 
from multiprocessing import Pool
import itertools

import utils 

# read config file
cfg = utils.read_config()
QP_list = cfg.qps.split()
output_dir = cfg.output_dir
recon_dir = cfg.recon_data_dir

# image directory
img_dir = '../dataset/validation/tracking'

# annotations
dataset_dir='../dataset'
annot_dir = os.path.join(dataset_dir, "annotations_tvd")

input_img_list = []
input_img_list.append(os.path.join(annot_dir, 'TVD-01.lst'))
input_img_list.append(os.path.join(annot_dir, 'TVD-02.lst'))
input_img_list.append(os.path.join(annot_dir, 'TVD-03.lst'))

output_data_file = os.path.join(output_dir, 'psnr_msssim_data.csv')
output_report_file  = os.path.join(output_dir, 'psnr_msssim_report.csv')


def process_img(img_id, seq_id):
  img_psnr_msssim_data = []
  img_fname = os.path.join(img_dir, seq_id, "img1", img_id+'.png')
  print(f'process {img_fname}') 
  img = cv2.imread(img_fname).astype(float) #HWC
  img_batch = torch.tensor(img).unsqueeze(0).permute(0,3,1,2).cuda()

  for QP in QP_list:
    recon_img_fname = os.path.join(recon_dir, QP, seq_id, "img1", img_id+'.png')
    recon_img = cv2.imread(recon_img_fname).astype(float) #HWC

    # calculate PSNR
    mse = np.mean((img - recon_img)**2)
    if mse == 0: 
      psnr = 100
    else:
      psnr = 20 * np.log10(255 / np.sqrt(mse))

    # calculate ssim and msssim
    recon_img_batch = torch.tensor(recon_img).unsqueeze(0).permute(0,3,1,2).cuda()
    ssim = pytorch_msssim.ssim(img_batch, recon_img_batch, data_range=255).item()
    msssim = pytorch_msssim.ms_ssim(img_batch, recon_img_batch, data_range=255).item()
    
    img_psnr_msssim_data.append({ \
      'ImageID': img_id,
      'SeqID': seq_id,
      'QP': QP,
      'PSNR': psnr,
      'SSIM': ssim,
      'MSSSIM': msssim})
  return img_psnr_msssim_data

if __name__ == '__main__':
    # get validation image ids
    val_img_ids = []
    for img_list_fname in input_img_list:
        with open(img_list_fname) as f:
            imgs = f.read().splitlines()
            ids = [os.path.splitext(x)[0] for x in imgs]
            val_img_ids.append(ids)


    with Pool(10) as p:
        psnr_msssim_data=[]
        for i in range(1,4):
            psnr_msssim_data += p.starmap(process_img, list(zip(val_img_ids[i-1], [f"TVD-0{i}"]*len(val_img_ids[i-1]))))

    psnr_msssim_data = list(itertools.chain(*psnr_msssim_data))

    psnr_msssim_data = pd.DataFrame(psnr_msssim_data)
    psnr_msssim_data.to_csv(output_data_file, index=False)
 

    psnr_msssim_data = pd.read_csv(output_data_file)
    report = []
    #detection
    for i in range(1,4):
        seq_id = f"TVD-0{i}"

        print('processing', seq_id)
  
        for QP in QP_list:
            sub_data = psnr_msssim_data.loc[(psnr_msssim_data['QP']==QP) & (psnr_msssim_data['SeqID']==seq_id)]
            #print(QP, sub_data)
            report.append({ \
                'SeqID': seq_id,
                'QP': QP,
                'PSNR': sub_data['PSNR'].mean(),
                'SSIM': sub_data['SSIM'].mean(),
                'MSSSIM': sub_data['MSSSIM'].mean()})

    report = pd.DataFrame(report)
    report.to_csv(output_report_file, index=False)

    print(report)


