# Copyright (c) 2020-2021 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
# This file is covered by the license agreement found in the file “license.txt” in the root of this project.

# evaluation according to the instruction from 
#https://github.com/tensorflow/models/blob/master/research/object_detection/g3doc/challenge_evaluation.md#object-detection-track

set -e
# load configuration
cfg_fname=$1
source $cfg_fname

dataset_dir=$(realpath ../dataset)

anno_dir=${dataset_dir}/annotations_tvd
HIERARCHY_FILE=$anno_dir/tvd_label_hierarchy.json
BOUNDING_BOXES=$anno_dir/tvd_detection_validation_bbox.csv
IMAGE_LABELS=$anno_dir/tvd_detection_validation_labels.csv
LABEL_MAP=$anno_dir/tvd_coco_label_map.pbtxt

# get object_detection 
TF_od_dir=$(python -c "import object_detection,os;print(os.path.dirname(object_detection.__file__))")

# evaluation

if [ $eval_uncompressed_data == 1 ]; then
  qps="uncmprd $qps"
fi

for QP in $qps; do
  echo processing ... $QP

  # detection results has this format: ImageID,LabelName,Score,XMin,XMax,YMin,YMax
  INPUT_PREDICTIONS=${output_dir}/output_detection_${QP}.txt.oid.txt
  OUTPUT_METRICS=${output_dir}/detection_${QP}_metric.txt
  
  python $TF_od_dir/metrics/oid_challenge_evaluation.py \
      --input_annotations_boxes=${BOUNDING_BOXES} \
      --input_annotations_labels=${IMAGE_LABELS} \
      --input_class_labelmap=${LABEL_MAP} \
      --input_predictions=${INPUT_PREDICTIONS} \
      --output_metrics=${OUTPUT_METRICS} \

done

echo 
echo 'Evaluation results: '
echo 

for QP in $qps; do
  OUTPUT_METRICS=${output_dir}/detection_${QP}_metric.txt
  rslt=$(head $OUTPUT_METRICS -n 1 | cut -d , -f 2)
  echo $rslt
done

echo Done!

