# Copyright (c) 2020-2021 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
# This file is covered by the license agreement found in the file “license.txt” in the root of this project.

# evaluation according to instruction from 
#https://github.com/tensorflow/models/blob/master/research/object_detection/g3doc/challenge_evaluation.md#object-detection-track

set -e

# load configuration
cfg_fname=$1
source $cfg_fname

dataset_dir=$(realpath ../dataset)

anno_dir=${dataset_dir}/annotations_tvd
segmentation_mask_dir=$anno_dir/tvd_validation_masks
BOUNDING_BOXES=$anno_dir/tvd_segmentation_validation_bbox.csv
IMAGE_LABELS=$anno_dir/tvd_segmentation_validation_labels.csv
INSTANCE_SEGMENTATIONS=$anno_dir/tvd_segmentation_validation_masks.csv
LABEL_MAP=$anno_dir/tvd_coco_label_map.pbtxt

# get TF object_detection path
TF_od_dir=$(python -c "import object_detection,os;print(os.path.dirname(object_detection.__file__))")

if [ $eval_uncompressed_data == 1 ]; then
  qps="uncmprd $qps"
fi

# evaluation
for QP in $qps; do
  echo processing ... $QP 

  INPUT_PREDICTIONS=${output_dir}/output_segmentation_${QP}.txt.oid.txt
  OUTPUT_METRICS=${output_dir}/segmentation_${QP}_metric.txt


  python -u $TF_od_dir/metrics/oid_challenge_evaluation.py \
      --input_annotations_boxes=${BOUNDING_BOXES} \
      --input_annotations_labels=${IMAGE_LABELS} \
      --input_class_labelmap=${LABEL_MAP} \
      --input_predictions=${INPUT_PREDICTIONS} \
      --input_annotations_segm=${INSTANCE_SEGMENTATIONS} \
      --output_metrics=${OUTPUT_METRICS} 
  

done

echo 
echo 'Evaluation results: '
echo 

for QP in $qps; do
  OUTPUT_METRICS=${output_dir}/segmentation_${QP}_metric.txt
  rslt=$(head $OUTPUT_METRICS -n 1 | cut -d , -f 2)
  echo $rslt
done

echo Done!

