# Copyright (c) 2020-2021 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
# This file is covered by the license agreement found in the file “license.txt” in the root of this project.

# generate results csv file and plot
import os
import pandas as pd
import matplotlib.pyplot as plt
import utils

# load configuration
cfg = utils.read_config()


bpp_fname = os.path.join(cfg.output_dir, 'bpp_report.csv')

# read bpp data
bpp_data = pd.read_csv(bpp_fname)

results = []
for task in cfg.task.split():
  for QP in cfg.qps.split(): 
    metric_fname = os.path.join(cfg.output_dir, f'{task}_{QP}_metric.txt')
    metric_data = pd.read_csv(metric_fname, names=['class', 'performance'])
    performance = metric_data.iloc[0]['performance']
    results.append({
     'Task': task,
     'QP': QP,
     'Bpp': bpp_data.loc[bpp_data['QP']==QP].iloc[0]['bpp'],
     'mAP': performance})

  if cfg.eval_uncompressed_data == '1': 
    # uncompressed
    metric_fname = os.path.join(cfg.output_dir, f'{task}_uncmprd_metric.txt')
    metric_data = pd.read_csv(metric_fname, names=['class', 'performance'])
    performance = metric_data.iloc[0]['performance']
    results.append({
       'Task': task,
       'QP': 'uncompressed',
       'Bpp': 0,
       'mAP': performance})
  
results = pd.DataFrame(results)
print(results)

output_fname = os.path.join(cfg.output_dir, 'results.csv')
results.to_csv(output_fname, index=False)

# generate RD curve
for task in cfg.task.split():
  plt.figure()
  if cfg.eval_uncompressed_data == '1': 
    plt.axhline( \
      results.loc[(results['Task']==task) & (results['QP']=='uncompressed')].iloc[0]['mAP'],
      color='r')
  task_data = results.loc[(results['Task']==task) & (results['QP']!='uncompressed')]
  plt.plot(task_data['Bpp'], task_data['mAP'], 'o-', color='b')
  plt.title(f'Rate-performance curve for {task}')
  plt.grid()
  plt.xlabel('BPP')
  plt.ylabel('mAP@0.5')
  plt.savefig(os.path.join(cfg.output_dir, f'rd_{task}.png'))
  #plt.show()



