# Copyright (c) 2020-2021 Nokia Corporation and/or its subsidiary(-ies). All rights reserved.
# This file is covered by the license agreement found in the file “license.txt” in the root of this project.

set -e

# load configuration
cfg_fname=$1
source $cfg_fname

# number of workers
n_workers=1

# machine task
task=segmentation

# dataset directory
dataset_dir=$(realpath ${output_dir}/rescaled)
anno_dir=$(realpath ../dataset/annotations_tvd)

input_fname=${anno_dir}/tvd_detection_input_new.lst

# tasks: 'id','data_dir','input_lst','cuda_device'
task_list=()
for qp in $qps; do
  task_list+=(\
    "${qp}","${dataset_dir}"/"${qp}","${input_fname}",${cuda_device} \
  )
done

# uncompressed data are stored in a different directory 
uncmprd_data_dir=$(realpath ../dataset/validation/object_det_seg)

if [ $eval_uncompressed_data == 1 ]; then
  task_list+=("uncmprd","${uncmprd_data_dir}","${input_fname}",${cuda_device})
fi

function process_task() {
	
  task=$1
  task_info=$2
  IFS=',' read task_id data_dir in_fname cuda_dev <<< "${task_info}"

  echo
  echo "Processing ${task_id} ..."

  in_fname=$(realpath $in_fname)
  out_fname=$(realpath ${output_dir}/output_${task}_${task_id}.txt)
  
  echo $task, $data_dir, $in_fname, $out_fname  
  pushd $(pwd)
  cd ../detectron2

  CUDA_VISIBLE_DEVICES=${cuda_dev} python detectron2_predict.py \
    --image_dir $data_dir \
    --task $task \
    --input_file $in_fname \
    --output_file $out_fname

  echo Creating OID expanded results...

  python cvt_detectron_coco_oid_tvd.py \
    --coco_output_file $out_fname \
    --oid_output_file ${out_fname}.oid.txt \
    --selected_classes ./data/selected_classes_tvd.txt
  popd
 
  echo "Task ${task_id} done. "
  echo 

}
export -f process_task

for task_info in "${task_list[@]}"; do
  echo $task_info
  process_task ${task} ${task_info}
	
done

echo "All done!"

