#
# This script generates all results for object detection and instance segmentation on the 
# TVD dataset

set -e

#####################################################
# handle command line arguments
if [ "$#" -ne 1 ]; then
  echo "
This script generates object detection and segmentation tasks for anchors or proposed technologies. To reproduce anchors, run
  $0 anchor.cfg

To generate machine task results for technologies for VCM, run 
  $0 proposal.cfg
" 
  exit 0
else
  cfg_fname=$1  
fi

# read config file
source $cfg_fname
mkdir -p $output_dir

#####################################################

# calcuate bpp
echo "Calculating bpp for each compressed data"
python calculate_bpp_tvd.py $cfg_fname


# rescle the images to original resolution
echo "Rescaling the images to original resolution"
python rescale_tvd.py $cfg_fname


echo
echo "#################################################"
echo

# detection task
if echo $task | grep -w -q 'detection'; then
  echo "processing detection ..."
  ./gen_detout_tvd.sh $cfg_fname
  ./eval_det_tvd.sh $cfg_fname
fi



echo
echo "#################################################"
echo

# segmentation task
if echo $task | grep -w -q 'segmentation'; then
  echo "processing segmentation ..."
  ./gen_segout_tvd.sh $cfg_fname
  ./eval_seg_tvd.sh $cfg_fname
fi

echo
echo "#################################################"
echo


# generate report csv and plots
echo "generating summary ..."
python gen_results.py $cfg_fname

echo 
echo 'Done!'

